import { NativeModules, NativeEventEmitter } from 'react-native';
const HBDEventEmitter = NativeModules.HBDEventEmitter;
const HBDEventReceiver = new NativeEventEmitter(HBDEventEmitter);
function listenComponentResult(listener) {
    const { EVENT_NAVIGATION, KEY_ON, ON_COMPONENT_RESULT, KEY_REQUEST_CODE, KEY_RESULT_CODE, KEY_RESULT_DATA, KEY_SCENE_ID, } = HBDEventEmitter.getConstants();
    return HBDEventReceiver.addListener(EVENT_NAVIGATION, data => {
        if (data[KEY_ON] === ON_COMPONENT_RESULT) {
            const requestCode = data[KEY_REQUEST_CODE];
            const resultCode = data[KEY_RESULT_CODE];
            const resultData = data[KEY_RESULT_DATA];
            const sceneId = data[KEY_SCENE_ID];
            listener(sceneId, requestCode, resultCode, resultData);
        }
    });
}
function listenComponentVisibility(listener) {
    const { EVENT_NAVIGATION, KEY_ON, ON_COMPONENT_APPEAR, ON_COMPONENT_DISAPPEAR, KEY_SCENE_ID } = HBDEventEmitter.getConstants();
    return HBDEventReceiver.addListener(EVENT_NAVIGATION, data => {
        if (data[KEY_ON] === ON_COMPONENT_APPEAR) {
            listener(data[KEY_SCENE_ID], 'visible');
        }
        if (data[KEY_ON] === ON_COMPONENT_DISAPPEAR) {
            listener(data[KEY_SCENE_ID], 'invisible');
        }
    });
}
function listenBarButtonItemClick(listener) {
    const { EVENT_NAVIGATION, KEY_ON, ON_BAR_BUTTON_ITEM_CLICK, KEY_ACTION, KEY_SCENE_ID } = HBDEventEmitter.getConstants();
    return HBDEventReceiver.addListener(EVENT_NAVIGATION, data => {
        if (data[KEY_ON] === ON_BAR_BUTTON_ITEM_CLICK) {
            listener(data[KEY_SCENE_ID], data[KEY_ACTION]);
        }
    }, {});
}
function listenTabSwitch(listener) {
    const { EVENT_SWITCH_TAB, KEY_INDEX, KEY_SCENE_ID } = HBDEventEmitter.getConstants();
    return HBDEventReceiver.addListener(EVENT_SWITCH_TAB, event => {
        const index = event[KEY_INDEX];
        const [from, to] = index.split('-');
        listener(event[KEY_SCENE_ID], parseInt(from, 10), parseInt(to, 10));
    });
}
function listenWillSetRoot(willSetRoot) {
    const { EVENT_WILL_SET_ROOT } = HBDEventEmitter.getConstants();
    return HBDEventReceiver.addListener(EVENT_WILL_SET_ROOT, _ => {
        willSetRoot();
    });
}
function listenDidSetRoot(didSetRoot) {
    const { EVENT_DID_SET_ROOT } = HBDEventEmitter.getConstants();
    return HBDEventReceiver.addListener(EVENT_DID_SET_ROOT, (event) => {
        didSetRoot(event.tag);
    });
}
export default {
    listenBarButtonItemClick,
    listenComponentResult,
    listenComponentVisibility,
    listenTabSwitch,
    listenWillSetRoot,
    listenDidSetRoot,
};
