import type React from 'react';
import { ComponentProvider } from 'react-native';
import type { BarButtonItem, DefaultOptions, NavigationOption, TabBarStyle, TabItemInfo, TitleItem } from './Options';
import type { BuildInLayout, Layout, Route, RouteGraph, RouteConfig } from './Route';
import { ResultType } from './handler/ResultEventHandler';
import { GlobalVisibilityEventListener, VisibilityEventListener } from './handler/VisibilityEventHandler';
export type { ResultType } from './handler/ResultEventHandler';
import type { DispatchParams, NavigationInterceptor } from './handler/DispatchCommandHandler';
export type { DispatchParams, NavigationInterceptor } from './handler/DispatchCommandHandler';
type BarButtonClickEventListener = (sceneId: string, value: Function) => void;
type Nullable<T> = {
    [P in keyof T]: T[P] extends T[P] | undefined ? T[P] | null : T[P];
};
export declare const RESULT_CANCEL: number, RESULT_OK: number, RESULT_BLOCK: number;
export type NavigationSubscription = {
    remove: () => void;
};
export type HOC = (WrappedComponent: React.ComponentType<any>) => React.ComponentType<any>;
export interface Navigation {
    startRegisterComponent(hoc?: HOC): void;
    endRegisterComponent(): void;
    registerComponent(appKey: string, getComponentFunc: ComponentProvider, routeConfig?: RouteConfig): void;
    routeConfigs(): Map<string, RouteConfig>;
    setNavigationComponentWrap(wrap: (moduleName: string) => HOC): void;
    addGlobalVisibilityEventListener(listener: GlobalVisibilityEventListener): NavigationSubscription;
    addVisibilityEventListener(sceneId: string, listener: VisibilityEventListener): NavigationSubscription;
    dispatch(sceneId: string, action: string, params?: DispatchParams): Promise<boolean>;
    setInterceptor(interceptor: NavigationInterceptor): void;
    setResult<T extends ResultType>(sceneId: string, resultCode: number, data?: T): void;
    result<T extends ResultType>(sceneId: string, resultCode: number): Promise<[number, T]>;
    unmount(sceneId: string): void;
    setRoot(layout: BuildInLayout | Layout, sticky?: boolean): Promise<boolean>;
    setRootLayoutUpdateListener(willSetRoot?: () => void, didSetRoot?: () => void): void;
    currentTab(sceneId: string): Promise<number>;
    findSceneIdByModuleName(moduleName: string): Promise<string | null>;
    isStackRoot(sceneId: string): Promise<boolean>;
    signalFirstRenderComplete(sceneId: string): void;
    currentRoute(): Promise<Route>;
    routeGraph(): Promise<RouteGraph[]>;
    setDefaultOptions(options: DefaultOptions): void;
    setLeftBarButtonItem(sceneId: string, buttonItem: Nullable<BarButtonItem> | null): void;
    setRightBarButtonItem(sceneId: string, buttonItem: Nullable<BarButtonItem> | null): void;
    setLeftBarButtonItems(sceneId: string, buttonItems: Array<Nullable<BarButtonItem>> | null): void;
    setRightBarButtonItems(sceneId: string, buttonItems: Array<Nullable<BarButtonItem>> | null): void;
    setTitleItem(sceneId: string, titleItem: TitleItem): void;
    updateOptions(sceneId: string, options: NavigationOption): void;
    updateTabBar(sceneId: string, options: TabBarStyle): void;
    setTabItem(sceneId: string, item: TabItemInfo | TabItemInfo[]): void;
    setMenuInteractive(sceneId: string, enabled: boolean): void;
    setBarButtonClickEventListener(listener: BarButtonClickEventListener): void;
}
declare const _default: Navigation;
export default _default;
