import React, { useEffect } from 'react';
import { Navigator, NavigationContext } from './Navigator';
import Navigation from './Navigation';
function getDisplayName(WrappedComponent) {
    return WrappedComponent.displayName || WrappedComponent.name || 'Component';
}
export function withNavigation(moduleName) {
    return function (WrappedComponent) {
        const FC = React.forwardRef((props, ref) => {
            const { sceneId } = props;
            const navigator = Navigator.of(sceneId);
            if (navigator.moduleName === undefined) {
                navigator.moduleName = moduleName;
            }
            useEffect(() => {
                Navigation.signalFirstRenderComplete(sceneId);
                return () => {
                    Navigator.unmount(sceneId);
                };
            }, [sceneId]);
            const injected = {
                navigator,
            };
            return (<NavigationContext.Provider value={navigator}>
						<WrappedComponent ref={ref} {...props} {...injected}/>
					</NavigationContext.Provider>);
        });
        FC.displayName = `withNavigation(${getDisplayName(WrappedComponent)})`;
        return FC;
    };
}
export function withNavigationItem(item) {
    return function (WrappedComponent) {
        let navigationItem = WrappedComponent.navigationItem;
        if (navigationItem) {
            WrappedComponent.navigationItem = {
                ...navigationItem,
                ...item,
            };
        }
        else {
            WrappedComponent.navigationItem = item;
        }
        return WrappedComponent;
    };
}
