import { createContext } from 'react';
import store from './store';
import Navigation, { RESULT_BLOCK } from './Navigation';
export const NavigationContext = createContext(null);
let tagGenerator = 0;
Navigation.addGlobalVisibilityEventListener((sceneId, visibility) => {
    const navigator = store.getNavigator(sceneId);
    if (navigator) {
        navigator.visibility = visibility;
    }
});
Navigation.setBarButtonClickEventListener((sceneId, value) => {
    const navigator = Navigator.of(sceneId);
    value(navigator);
});
class Navigator {
    sceneId;
    moduleName;
    static of(sceneId) {
        let navigator = store.getNavigator(sceneId);
        if (!navigator) {
            navigator = new Navigator(sceneId);
            store.addNavigator(sceneId, navigator);
        }
        return navigator;
    }
    static unmount = (sceneId) => {
        store.removeNavigator(sceneId);
        Navigation.unmount(sceneId);
    };
    static async find(moduleName) {
        const sceneId = await Navigation.findSceneIdByModuleName(moduleName);
        if (sceneId) {
            return Navigator.of(sceneId);
        }
    }
    static async current() {
        const route = await Navigation.currentRoute();
        return Navigator.of(route.sceneId);
    }
    constructor(sceneId, moduleName) {
        this.sceneId = sceneId;
        this.moduleName = moduleName;
    }
    visibility = 'pending';
    state = {
        params: {},
    };
    setParams = (params) => {
        this.state.params = { ...this.state.params, ...params };
    };
    push = async (moduleName, props = {}, options = {}) => {
        const success = await Navigation.dispatch(this.sceneId, 'push', {
            moduleName,
            from: this.moduleName,
            to: moduleName,
            props,
            options,
        });
        if (!success) {
            return [RESULT_BLOCK, null];
        }
        return Navigation.result(this.sceneId, 0);
    };
    pushLayout = async (layout) => {
        const success = await Navigation.dispatch(this.sceneId, 'pushLayout', {
            from: this.moduleName,
            layout,
        });
        if (!success) {
            return [RESULT_BLOCK, null];
        }
        return Navigation.result(this.sceneId, 0);
    };
    pop = () => Navigation.dispatch(this.sceneId, 'pop', { from: this.moduleName });
    popTo = (moduleName, inclusive = false) => Navigation.dispatch(this.sceneId, 'popTo', {
        moduleName,
        inclusive,
        from: this.moduleName,
    });
    popToRoot = () => Navigation.dispatch(this.sceneId, 'popToRoot', { from: this.moduleName });
    redirectTo = (moduleName, props = {}, options = {}) => Navigation.dispatch(this.sceneId, 'redirectTo', {
        moduleName,
        from: this.moduleName,
        to: moduleName,
        props,
        options,
    });
    present = async (moduleName, props = {}, options = {}) => {
        const requestCode = --tagGenerator;
        const success = await Navigation.dispatch(this.sceneId, 'present', {
            moduleName,
            from: this.moduleName,
            to: moduleName,
            props,
            options,
            requestCode,
        });
        if (!success) {
            console.warn(`present from:${this.moduleName} to:${moduleName} failed. Maybe the screen (${this.moduleName}) already shows or present another screen.`);
            return [RESULT_BLOCK, null];
        }
        return Navigation.result(this.sceneId, requestCode);
    };
    presentLayout = async (layout) => {
        const requestCode = --tagGenerator;
        const success = await Navigation.dispatch(this.sceneId, 'presentLayout', {
            layout,
            from: this.moduleName,
            requestCode,
        });
        if (!success) {
            console.warn(`presentLayout from:${this.moduleName} failed. Maybe the screen (${this.moduleName}) already shows or present another screen.`);
            return [RESULT_BLOCK, null];
        }
        return Navigation.result(this.sceneId, requestCode);
    };
    dismiss = () => Navigation.dispatch(this.sceneId, 'dismiss', { from: this.moduleName });
    showModal = async (moduleName, props = {}, options = {}) => {
        const requestCode = --tagGenerator;
        const success = await Navigation.dispatch(this.sceneId, 'showModal', {
            moduleName,
            from: this.moduleName,
            to: moduleName,
            props,
            options,
            requestCode,
        });
        if (!success) {
            console.warn(`showModal from:${this.moduleName} to:${moduleName} failed. Maybe the screen (${this.moduleName}) already shows or present another screen.`);
            return [RESULT_BLOCK, null];
        }
        return Navigation.result(this.sceneId, requestCode);
    };
    showModalLayout = async (layout) => {
        const requestCode = --tagGenerator;
        const success = await Navigation.dispatch(this.sceneId, 'showModalLayout', {
            from: this.moduleName,
            layout,
            requestCode,
        });
        if (!success) {
            console.warn(`showModal from:${this.moduleName} failed. Maybe the screen (${this.moduleName}) already shows or present another screen.`);
            return [RESULT_BLOCK, null];
        }
        return Navigation.result(this.sceneId, requestCode);
    };
    hideModal = () => Navigation.dispatch(this.sceneId, 'hideModal', { from: this.moduleName });
    setResult = (resultCode, data = null) => Navigation.setResult(this.sceneId, resultCode, data);
    switchTab = async (index, popToRoot = false) => {
        const from = await Navigation.currentTab(this.sceneId);
        return Navigation.dispatch(this.sceneId, 'switchTab', {
            from,
            to: index,
            popToRoot,
        });
    };
    toggleMenu = () => Navigation.dispatch(this.sceneId, 'toggleMenu', {
        from: this.moduleName,
    });
    openMenu = () => Navigation.dispatch(this.sceneId, 'openMenu', { from: this.moduleName });
    closeMenu = () => Navigation.dispatch(this.sceneId, 'closeMenu', { from: this.moduleName });
    isStackRoot = () => Navigation.isStackRoot(this.sceneId);
}
export { Navigator };
