import type { Insets } from 'react-native';
import type { Navigator } from './Navigator';
export type Color = string;
export type ImageSource = {
    uri: string;
    scale?: number;
    height?: number;
    width?: number;
};
export declare const BarStyleLightContent = "light-content";
export type BarStyleLightContent = typeof BarStyleLightContent;
export declare const BarStyleDarkContent = "dark-content";
export type BarStyleDarkContent = typeof BarStyleDarkContent;
export type BarStyle = BarStyleLightContent | BarStyleDarkContent;
export declare const TitleAlignmentLeft = "left";
export type TitleAlignmentLeft = typeof TitleAlignmentLeft;
export declare const TitleAlignmentCenter = "center";
export type TitleAlignmentCenter = typeof TitleAlignmentCenter;
export type TitleAlignment = TitleAlignmentCenter | TitleAlignmentLeft;
export declare const LayoutFittingExpanded = "expanded";
export type LayoutFittingExpanded = typeof LayoutFittingExpanded;
export declare const LayoutFittingCompressed = "compressed";
export type LayoutFittingCompressed = typeof LayoutFittingCompressed;
export type LayoutFitting = LayoutFittingExpanded | LayoutFittingCompressed;
export interface NavigationOption {
    screenBackgroundColor?: Color;
    statusBarHidden?: boolean;
    statusBarColorAndroid?: Color;
    topBarStyle?: BarStyle;
    topBarColor?: Color;
    topBarShadowHidden?: boolean;
    topBarAlpha?: number;
    topBarTintColor?: Color;
    titleTextColor?: Color;
    titleTextSize?: number;
    navigationBarColorAndroid?: Color;
    navigationBarHiddenAndroid?: boolean;
    fitsOpaqueNavigationBarAndroid?: boolean;
    displayCutoutWhenLandscapeAndroid?: boolean;
    homeIndicatorAutoHiddenIOS?: boolean;
    backButtonHidden?: boolean;
    backInteractive?: boolean;
}
export interface NavigationItem extends NavigationOption {
    passThroughTouches?: boolean;
    forceTransparentDialogWindow?: boolean;
    animatedTransition?: boolean;
    forceScreenLandscape?: boolean;
    topBarHidden?: boolean;
    extendedLayoutIncludesTopBar?: boolean;
    swipeBackEnabled?: boolean;
    titleItem?: TitleItem;
    leftBarButtonItem?: BarButtonItem;
    rightBarButtonItem?: BarButtonItem;
    leftBarButtonItems?: BarButtonItem[];
    rightBarButtonItems?: BarButtonItem[];
    backItemIOS?: BackItem;
    tabItem?: TabItem;
}
export interface BarButtonItem {
    title?: string;
    icon?: ImageSource;
    insetsIOS?: Insets;
    action?: (navigator: Navigator) => void;
    enabled?: boolean;
    tintColor?: Color;
    renderOriginal?: boolean;
}
export interface TitleItem {
    title?: string;
    moduleName?: string;
    layoutFitting?: LayoutFitting;
}
export interface BackItem {
    title: string;
    tintColor?: Color;
}
export interface TabItem {
    title: string;
    icon?: ImageSource;
    unselectedIcon?: ImageSource;
    hideTabBarWhenPush?: boolean;
}
export interface ShadowImage {
    image?: ImageSource;
    color?: Color;
}
export interface DefaultOptions {
    screenBackgroundColor?: Color;
    topBarStyle?: BarStyle;
    topBarColor?: Color;
    topBarColorDarkContent?: Color;
    topBarColorLightContent?: Color;
    statusBarColorAndroid?: Color;
    navigationBarColorAndroid?: Color;
    hideBackTitleIOS?: boolean;
    elevationAndroid?: number;
    shadowImage?: ShadowImage;
    backIcon?: ImageSource;
    topBarTintColor?: Color;
    topBarTintColorDarkContent?: Color;
    topBarTintColorLightContent?: Color;
    titleTextColor?: Color;
    titleTextColorDarkContent?: Color;
    titleTextColorLightContent?: Color;
    titleTextSize?: number;
    titleAlignmentAndroid?: TitleAlignment;
    barButtonItemTextSize?: number;
    swipeBackEnabledAndroid?: boolean;
    splitTopBarTransitionIOS?: boolean;
    scrimAlphaAndroid?: number;
    fitsOpaqueNavigationBarAndroid?: boolean;
    displayCutoutWhenLandscapeAndroid?: boolean;
    tabBarColor?: Color;
    tabBarShadowImage?: ShadowImage;
    tabBarItemColor?: Color;
    tabBarUnselectedItemColor?: Color;
    tabBarBadgeColor?: Color;
}
export type TabBarStyle = Pick<DefaultOptions, 'tabBarColor' | 'tabBarShadowImage' | 'tabBarItemColor' | 'tabBarUnselectedItemColor'>;
export interface TabItemInfo {
    index: number;
    title?: string;
    badge?: {
        text?: string;
        hidden: boolean;
        dot?: boolean;
    };
    icon?: {
        selected: ImageSource;
        unselected?: ImageSource;
    };
}
