import Event from '../Event';
export default class ResultEventHandler {
    listenerRecords = {};
    constructor() { }
    handleComponentResult() {
        Event.listenComponentResult((sceneId, requestCode, resultCode, data) => {
            const listeners = this.listenerRecords[sceneId]?.[requestCode];
            if (listeners) {
                delete this.listenerRecords[sceneId][requestCode];
                listeners.forEach(l => l(resultCode, data));
            }
        });
    }
    addResultEventListener(sceneId, requestCode, listener) {
        if (!this.listenerRecords[sceneId]) {
            this.listenerRecords[sceneId] = {};
        }
        if (!this.listenerRecords[sceneId][requestCode]) {
            this.listenerRecords[sceneId][requestCode] = [];
        }
        this.listenerRecords[sceneId][requestCode].push(listener);
    }
    removeResultEventListener(sceneId, requestCode) {
        if (this.listenerRecords[sceneId]) {
            delete this.listenerRecords[sceneId][requestCode];
        }
    }
    invalidateResultEventListener(sceneId) {
        if (this.listenerRecords[sceneId]) {
            delete this.listenerRecords[sceneId];
        }
    }
    waitResult(sceneId, requestCode) {
        return new Promise(resolve => {
            const listener = (resultCode, data) => {
                this.removeResultEventListener(sceneId, requestCode);
                resolve([resultCode, data]);
            };
            this.addResultEventListener(sceneId, requestCode, listener);
        });
    }
}
